#ifndef _FLE_MDIWINDOW_H
#define _FLE_MDIWINDOW_H

#include "FLE_Workspace.h"
#include "FLE_Boxtypes.h"

#include "FLE_Util.h"

#include <fltk/Fl_Group.h>
#include <fltk/Fl_Window.h>
#include <fltk/Fl_Box.h>
#include <fltk/Fl_Button.h>
#include <fltk/Fl_Pixmap.h>
#include <fltk/Fl.h>

#include <string.h>

#define NO_RESIZE	 -1
#define TOP					0
#define BOTTOM			1
#define RIGHT				2
#define LEFT				3
#define RIGHTTOP    4
#define RIGHTBOTTOM 5
#define LEFTTOP     6
#define LEFTBOTTOM  7

class MDI_Menu_Add;
class FLE_MdiWindow;

class FLE_MdiTitlebar : public Fl_Group {
    friend class FLE_MdiWindow;
public:
    FLE_MdiTitlebar(int x,int y,int w,int h,const char *l=0);

    static char *FLE_MdiTitlebar::get_line(const char *buf, int maxwidth);

    virtual void draw();
    virtual int handle(int event);
    virtual void layout();

    static int ex,ey;
    static int old_rx,old_ry;

		Fl_Button *close() { return &_close; }

protected:
    Fl_Button _close, _max, _min;
    FLE_MdiWindow *_owner;
};

class FLE_MdiWindow : public Fl_Window {
    friend class FLE_MdiViewport;
	friend class FLE_WorkSpace;
    friend class FLE_MdiTitlebar;
    friend class MDI_Menu_Add;
public:
    FLE_MdiWindow(int x, int y, int w, int h, const char *label);
    virtual ~FLE_MdiWindow();

	bool active() { return _active; }
	void active(bool a) { _active = a; }
		
    // Set/get view
    Fl_Group *view(Fl_Group *v); //returns old view
    Fl_Group *view() { return prv; }

    void resizable(Fl_Widget *w) { prv->resizable(w); }
    void resizable(Fl_Widget &w) { prv->resizable(w); }
    Fl_Widget *resizable()       { return prv->resizable(); }

    // Caption functions
    void caption(const char *cap) { if(cap) strncpy(_cap, cap, 4096); else _cap[0]='\0'; _titlebar.redraw(); }
    void label(const char *_label) { caption(_label); }
    const char *caption() { return _cap; }
    const char *label() { return caption(); }

    //void titlebar(Fl_Box *b) { _titlebar = b; }
    FLE_MdiTitlebar* titlebar() {return &_titlebar;}

    bool toplevel() { return _toplevel; }

    void detach();
    void attach() { attach(_owner); }
    void attach(FLE_MdiViewport *ws);

    void owner(FLE_MdiViewport *ws) { _owner = ws; }
    FLE_MdiViewport *owner() { return _owner; }

    void close_callback();
    void minimize(bool val);

    bool minimized() { return _minimized; }

    void setTop() { if(!_toplevel && _owner) _owner->top(this); else show(); }
    bool isTop() { return _toplevel==true ? 0 : (_owner->top()==this); }

    void maximize(bool val);
    bool maximized() { return _maximized; }

    void animate_mode(bool o) { _anim_opaque = o; }
    bool animate_mode() { return _anim_opaque; }
    void animate(bool val) { _animate = val; }
    bool animate() { return _animate; }

    void boundary_check(bool val) { _boundaries = val; }
    bool boundary_check() { return _boundaries; }

    int minw() { return _minw; }
    int minh() { return _minh; }
    void minw(int w) { _minw = w+box()->dw(); }
    void minh(int h) { _minh = h+box()->dw()+titlebar()->h(); }

protected:
    virtual int handle(int ev);
    virtual void handle_resize(int where);
    virtual void draw();
    virtual void layout();

private:
    void check_move_boundary(int &x, int &y);
    void check_size_boundary(int &w, int &h);

    void animate_min(int mx, int my, int mw, int mh);
    void animate_max(int mx, int my, int mw, int mh);

    void _resize(int x, int y, int w, int h);
    void _position(int x, int y);
    void minmax();

    FLE_MdiViewport *_owner; //Workspace who own this window.
    Fl_Group *prv;
    FLE_MdiTitlebar _titlebar;

    char _cap[4096];

    bool _toplevel;
	bool _active;

    void h_oldsize(int w, int h) { _how = w; _hoh = h; }
    void h_oldpos(int x, int y)  { _hox = x; _hoy = y; }
    int _hox, _hoy, _how, _hoh;

    void oldsize(int w, int h) { _ow = w; _oh = h; }
    void oldpos(int x, int y)  { _ox = x; _oy = y; }
    int _ox, _oy, _oh, _ow;

    bool _boundaries;
    bool _animate;
    bool _maximized;
    bool _minimized;

    int _resize_where;

    int tx,ty,tw,th; //Box for titlebar
    int px,py,pw,ph; //Box for view

    FLE_Rect bot;
    FLE_Rect top;
    FLE_Rect left;
    FLE_Rect right;

    FLE_Rect ltcorn;
    FLE_Rect rtcorn;
    FLE_Rect lbcorn;
    FLE_Rect rbcorn;

    bool _anim_opaque;

    short _minw, _minh;
    short _maxw, _maxh;

    MDI_Menu_Add *menuadd;
    void create_ma();
    void delete_ma();
};

#endif

