/***************************************************************************
                          FLE_String.h  -  description
                             -------------------
    begin                : Tue Dec 14 1999
    copyright            : (C) 1999 by Alexey Parshin
    email                : sparx@olma.co.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 /*  modified by Martin Pekar 01/04/2002 */
#ifndef __FLE_STRING_H__
#define __FLE_STRING_H__

#include "EDE.h"
#include <string.h>
#include <stdlib.h>

class FLE_String;

extern EDE_API bool operator <  (const FLE_String &s1, const FLE_String &s2);
extern EDE_API bool operator <  (const FLE_String &s1, const FLE_String &s2);
extern EDE_API bool operator <= (const FLE_String &s1, const FLE_String &s2);
extern EDE_API bool operator >  (const FLE_String &s1, const FLE_String &s2);
extern EDE_API bool operator >= (const FLE_String &s1, const FLE_String &s2);
extern EDE_API bool operator == (const FLE_String &s1, const FLE_String &s2);
extern EDE_API bool operator != (const FLE_String &s1, const FLE_String &s2);

extern EDE_API bool operator <  (const FLE_String &s1, const char *s2);
extern EDE_API bool operator <= (const FLE_String &s1, const char *s2);
extern EDE_API bool operator >  (const FLE_String &s1, const char *s2);
extern EDE_API bool operator >= (const FLE_String &s1, const char *s2);
extern EDE_API bool operator == (const FLE_String &s1, const char *s2);
extern EDE_API bool operator != (const FLE_String &s1, const char *s2);

class EDE_API FLE_String {
   friend EDE_API FLE_String operator +(const char*, const FLE_String& rhs);
protected:
   char *str;
   void assign(const char *st);

public:
   // constructors & destructors
   FLE_String(char c,int repeater = 1);
   FLE_String(int number);
   FLE_String(unsigned number);
   FLE_String(const char *s="") { if(!s) s=""; str = strdup(s); }
   FLE_String(const char *s, int maxlen);
   FLE_String(const FLE_String& s) { str = strdup(s.str); }
   ~FLE_String() { delete []str; }

   // assignments
   FLE_String& operator = (const char *s);
   FLE_String& operator = (const FLE_String& s);

   // comparisions
   friend EDE_API bool operator <  (const FLE_String &s1, const FLE_String &s2);
   friend EDE_API bool operator <= (const FLE_String &s1, const FLE_String &s2);
   friend EDE_API bool operator >  (const FLE_String &s1, const FLE_String &s2);
   friend EDE_API bool operator >= (const FLE_String &s1, const FLE_String &s2);
   friend EDE_API bool operator == (const FLE_String &s1, const FLE_String &s2);
   friend EDE_API bool operator != (const FLE_String &s1, const FLE_String &s2);

   bool cmp(FLE_String &s);
   bool casecmp(FLE_String &s);

   // comparisions with regular strings
   friend EDE_API bool operator <  (const FLE_String &s1, const char *s2);
   friend EDE_API bool operator <= (const FLE_String &s1, const char *s2);
   friend EDE_API bool operator >  (const FLE_String &s1, const char *s2);
   friend EDE_API bool operator >= (const FLE_String &s1, const char *s2);
   friend EDE_API bool operator == (const FLE_String &s1, const char *s2);
   friend EDE_API bool operator != (const FLE_String &s1, const char *s2);

   // operations
   FLE_String  operator +  (const char * s) const;
   FLE_String  operator +  (const FLE_String& s) const;
   FLE_String& operator += (const char * s);
   FLE_String& operator += (const FLE_String& s);

   // usefull methods
   void clear();
   FLE_String  trimRight() const;
   FLE_String  trimLeft()  const;
   FLE_String  trim()      const;
   FLE_String  remove(const char *pattern) const;
   FLE_String  replace(const char *pattern,const char *replacement) const;
   FLE_String  lowerCase() const;
   FLE_String  upperCase() const;
   FLE_String  subString(int start,int count) const;
   void     subDelete(int start,int count=1);
   void     subDelete(const char *s) { subReplace(s,NULL); }
   void     subInsert(int start,const char *str);
   void     subInsert(int start,char ch);
   void     subReplace(const char *s_str,const char *r_str);
   FLE_String  wordAt(long position) const;
   FLE_String& wrapAt(long limit, FLE_String *remainder=NULL);
   FLE_String& chopAt(long limit, FLE_String *remainder=NULL);
   int      length() const;
   int      pos(const char *substr, int index=0) const;
   int      pos(int c, int index=0) const;
   int      rpos(int c, int index=0) const;
   int      toInt(int defvalue=0) const;
   double   toFloat(float defvalue=0) const;
   bool     isEmpty() const;
   void     setLength(int newLength);
   FLE_String  substituteSpecialChars(char prefix) const;
   const char *c_str() const { return str; }

   void printf(const char *str, ...);

   // auto conversion to char* when needed    
   operator const char *() { return str; }   
   operator char *() { return str; }   
   
   // element access
   char& operator [] (const int i) { return str[i]; }
   char  operator [] (const int i) const { return str[i]; }

   // extra functionality to support CEditor
   char& lastCharacter() const { return str[strlen(str)-1]; };

   static FLE_String IntToString(int val);
};

#endif
